/**
 ** locoDemoCntrlTaskCodels.c
 **
 ** Codels used by the control task locoDemoCntrlTask
 **
 ** Author: 
 ** Date: 
 **
 **/

#include <math.h>
#include <portLib.h>

#include "locoDemoConst.h"
#include "server/locoDemoHeader.h"


/*------------------------------------------------------------------------
 * recalConfig  -  control codel of CONTROL request SetConfig
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
recalConfig(GENPOS_CART_STATE *config, int *report)
{
  GENPOS_CART_REF *refId = &locoDemoDataStrId->ref;

  /* Nouvelle consigne */
  refId->x = config->xRob;
  refId->y = config->yRob;
  refId->v = 0.;
  refId->w = 0.;

  /* On recale la position */
  locoDemoDataStrId->robot = *config;

  return(OK);
}

/*------------------------------------------------------------------------
 * controlRef  -  control codel of CONTROL request SetRef
 *
 * Description:    
 * Report: OK
 *                 S_locoDemo_BAD_REF_DATA
 *
 * Returns:    OK or ERROR
 */

STATUS
controlRef(GENPOS_CART_REF *ref, int *report)
{
  /* Verifier la validite des arguments */
  if (ref->v != 0. || ref->w != 0. || 
      (ref->backFlag != GENPOS_FORWARD_MOTION 
	  && ref->backFlag != GENPOS_BACKWARD_MOTION)) {
    *report = S_locoDemo_BAD_REF_DATA;
    return(ERROR);
  }

  return(OK);
}

/*------------------------------------------------------------------------
 * controlCmdParam  -  control codel of CONTROL request SetCmdParam
 *
 * Description:    
 * Report: OK
 *                 S_locoDemo_BAD_CMD_PARAMETERS
 *
 * Returns:    OK or ERROR
 */

STATUS
controlCmdParam(GENPOS_CART_CMD_PARAM *cmd, int *report)
{
  /* Verifier la validite des arguments */
  if (cmd->KpLongit <= 0. || cmd->KpTransv <= 0.) {
    *report = S_locoDemo_BAD_CMD_PARAMETERS;
    return (ERROR);
  }

  return(OK);
}

/*------------------------------------------------------------------------
 * computStopRef  -  control codel of CONTROL request Stop
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
computStopRef(int *report)
{
  GENPOS_CART_STATE *robotId;		/* Config robot */
  GENPOS_CART_REF *refId;		/* Consigne */
  double dtArret, dtRetard;  /* Temps prise en compte et temps deceleration */
  double dSArret, dTArret;   /* Longueurs lin et ang de decel */
  double angleFinal;         /* Angle a l'arrivee */
  double period;

  robotId = &locoDemoDataStrId->robot;
  refId = &locoDemoDataStrId->ref;

  /* Longueur d'arret */
  period = EXEC_TASK_PERIOD(LOCODEMO_MOTIONTASK_NUM);
  dtArret = MAX (fabs(robotId->v) / LOCODEMO_ACCEL_LIN_MAX,
		 fabs(robotId->w) / LOCODEMO_ACCEL_ANG_MAX);
  dtRetard =  1 * period;
  dSArret = robotId->v * (dtArret / 2 + dtRetard);
  dTArret = robotId->w * (dtArret / 2 + dtRetard);
  angleFinal = robotId->theta + dTArret;

  /* Consigne */
  refId->x = robotId->xRob + dSArret * cos (angleFinal);
  refId->y = robotId->yRob + dSArret * sin (angleFinal);
  refId->v = 0. ;
  refId->w = 0. ;
  return(OK);
}

/*------------------------------------------------------------------------
 * trackStart  -  control codel of EXEC request Track
 *
 * Description:    
 * Report: OK
 *                 S_locoDemo_POSTER_NOT_FIND
 *                 S_locoDemo_BAD_REF
 *                 S_locoDemo_CMD_STOP_TRACK
 *
 * Returns:    OK or ERROR
 */

STATUS
trackStart(GENPOS_POSTER_NAME *trackedPoster, int *report)
{
  /* Debloquer le tracking */
  locoDemoDataStrId->trackStr.trackMode = GENPOS_TRACK_CONFIG;
  return (OK);
}


